/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of FRETSaw.
 *
 *    FRETSaw is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    FRETSaw is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.FretSaw;

import java.awt.Color;

import uk.ac.bbsrc.babraham.FretSaw.Display.ColourGradient;
import uk.ac.bbsrc.babraham.FretSaw.Preferences.FretSawPreferences;

import ij.ImagePlus;
import ij.gui.NewImage;
import ij.process.ImageProcessor;

public class FretImageCreator {
	
	public static ImagePlus createFretImage (ImagePlus channel1, ImagePlus channel2) {
	
		
		FretSawPreferences prefs = FretSawPreferences.getInstance();
		
		ImagePlus fretImage = NewImage.createRGBImage("Fret", channel1.getWidth()+30, channel1.getHeight(), channel1.getStackSize(), NewImage.FILL_BLACK);
		
		// Now we need to go through all of the pixels in the 
		// original image and work out the FRET score we
		// want to use.
		
		// Find out what cutoff we want to use.  This will either be a specified
		// fixed value, or an automatically calculated threshold
		
		int minIntensity1 = 0;
		int minIntensity2 = 0;
		
		if (prefs.useAutoScale()) {
			minIntensity1 = channel1.getProcessor().getAutoThreshold();
			minIntensity2 = channel2.getProcessor().getAutoThreshold();
		}
		else {
			minIntensity1 = prefs.minIntensity();
			minIntensity2 = prefs.minIntensity();
		}
				
		for (int slice=1;slice<=channel1.getStackSize();slice++) {
//			System.out.println("Making slice "+slice);
			channel1.setSlice(slice);
			channel2.setSlice(slice);
			fretImage.setSlice(slice);
			ImageProcessor p1 = channel1.getProcessor();
			ImageProcessor p2 = channel2.getProcessor();
			ImageProcessor f = fretImage.getProcessor();
			
			for (int x=0;x<p1.getWidth();x++) {
				for (int y=0;y<p1.getHeight();y++) {
					
					int p1v = getIntensity(p1,x,y,prefs.smoothing());
					int p2v = getIntensity(p2, x, y, prefs.smoothing());
										
					double ratio = (p2v-minIntensity2)/(double)(p1v-minIntensity1);
					
					// We allow ratios < 1 even though the scaling will be
					// different to ratios > 1
					
					// Limit the maxRatio to the limit the user has set
					if (ratio > prefs.maxRatio()) ratio = prefs.maxRatio();
					
					Color c;

					// We now see if we want to keep this pixel
					
					// First we check if this is something where we're going to show a coloured FRET score
					if ((p1v > minIntensity1 || p2v > minIntensity2) && ratio > prefs.minRatio()) {
						c = ColourGradient.getColor(ratio, prefs.minRatio(), prefs.maxRatio());
					}
					
					// If not we will still show it (without colour) if they've opted to overlay all pixels
					else {
						if (prefs.addOverlay()) {
							c = Color.WHITE;
						}
						else {
							continue;
						}
					
					}
					
					// Having set the colour we now adjust the brightness to represent
					// the original intensities in the image;
					float [] hsbValues = new float[3];
					Color.RGBtoHSB(c.getRed(),c.getGreen(),c.getBlue(), hsbValues);
					
					/*
					 * We scale the intensity to the range used in each of the source
					 * images.  This is because the image formats allow a min and max
					 * value to be specified which may be less than the full theoretical
					 * intensity range of the format.  If we therefore scale to the full
					 * range we end up with really dim images.
					 */
					
					hsbValues[2] = (float)(Math.max((p1v-p1.getMin())/(p1.getMax()-p1.getMin()),(p2v-p2.getMin())/(p2.getMax()-p2.getMin())));

					// We also adjust the brightness by the percentage the user specified
					hsbValues[2] *= (prefs.brightness()/100f);
					
					c = Color.getHSBColor(hsbValues[0], hsbValues[1], hsbValues[2]);
					
					// Finally we draw this pixel on the new image.
					f.setColor(c);
					f.drawPixel(x+30, y);
					
				}
			}
			
			// Draw a scale bar on the left
			for (int y=0;y<p1.getHeight();y++) {
				f.setColor(ColourGradient.getColor(p1.getHeight()-y, 0, p1.getHeight()));
				f.drawLine(0, y, 30, y);
			}
			
			f.setColor(Color.WHITE);
			f.drawString(""+prefs.minRatio(), 2, p1.getHeight()-2);
			f.drawString(""+prefs.maxRatio(), 2, 17);
			
		}
		
		
		return fretImage;
	}
	
	private static int getIntensity (ImageProcessor p, int x, int y, int s) {
		
		int total = 0;
		int count = 0;
		
		for (int x1=x-s;x1<=x+s;x1++) {
			
			if (x1<0 || x1>=p.getWidth()) continue;
			
			for (int y1=y-s;y1<=y+s;y1++) {
				
				if (y1<0 || y1 >= p.getHeight()) continue;
				
				count++;
				total += p.get(x1,y1);
			
			}
		}
		
		return total/count;
	}
	
}
