/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of FRETSaw.
 *
 *    FRETSaw is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    FRETSaw is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.FretSaw.IO;

import ij.ImagePlus;

import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;

import uk.ac.bbsrc.babraham.FretSaw.FretSawApplication;
import uk.ac.bbsrc.babraham.FretSaw.Preferences.FretSawPreferences;

public class TIFFLoader extends ImageLoader {

	private TIFFLoaderDialog dialog;
	
	public TIFFLoader (FretSawApplication application) {
		super(application);
		dialog = new TIFFLoaderDialog();
	}
	
	@Override
	public String getFileName() {
		return dialog.fileName();
	}

	@Override
	public ImageSet getImageSet() throws IOException {
		dialog.setVisible(true);
		
		// This will block here since the dialog is modal
		
		return dialog.getImageSet();
	}

	@Override
	public File getLastDirectory() {
		// TODO Auto-generated method stub
		return null;
	}

	private class TIFFLoaderDialog extends JDialog implements ActionListener {
		
		private JTextField cfpField;
		private JTextField yfpField;
		private File cfpFile;
		private File yfpFile;
		private boolean cancelled = false;
		
		public TIFFLoaderDialog () {
			super(application,"Load TIFF Files");
			setModal(true);
			Container c = getContentPane();
			
			c.setLayout(new GridBagLayout());
			GridBagConstraints gbc = new GridBagConstraints();
			gbc.gridx=1;
			gbc.gridy=1;
			gbc.weightx=0.1;
			gbc.weighty=0.5;
			gbc.fill = GridBagConstraints.HORIZONTAL;
			
			c.add(new JLabel("CFP File",JLabel.LEFT),gbc);

			gbc.gridx++;
			gbc.weightx=0.8;
			cfpField = new JTextField();
			cfpField.setEditable(false);
			cfpField.setEnabled(false);
			c.add(cfpField,gbc);
			
			gbc.gridx++;
			gbc.weightx=0.1;
			JButton cfpButton = new JButton("Select");
			cfpButton.setActionCommand("select_cfp");
			cfpButton.addActionListener(this);
			c.add(cfpButton,gbc);
			

			gbc.gridx=1;
			gbc.gridy++;
			gbc.weightx=0.1;
			
			c.add(new JLabel("YFP File",JLabel.LEFT),gbc);

			gbc.gridx++;
			gbc.weightx=0.8;
			yfpField = new JTextField();
			yfpField.setEditable(false);
			yfpField.setEnabled(false);
			c.add(yfpField,gbc);
			
			gbc.gridx++;
			gbc.weightx=0.1;
			JButton yfpButton = new JButton("Select");
			yfpButton.setActionCommand("select_yfp");
			yfpButton.addActionListener(this);
			c.add(yfpButton,gbc);	

			gbc.gridx=1;
			gbc.gridy++;
			gbc.gridwidth = 3;
			gbc.weighty = 0.01;
			
			JPanel buttonPanel = new JPanel();
			JButton cancelButton = new JButton("Cancel");
			cancelButton.setActionCommand("cancel");
			cancelButton.addActionListener(this);
			buttonPanel.add(cancelButton);
			
			JButton loadButton = new JButton("Load");
			loadButton.setActionCommand("load");
			loadButton.addActionListener(this);
			buttonPanel.add(loadButton);
			
			c.add(buttonPanel,gbc);
			
			
			setSize(300,120);
			setLocationRelativeTo(application);
			
		}
		
		public ImageSet getImageSet () throws IOException {
			if (cancelled) return null;
			
			if (cfpFile == null || yfpFile == null) {
				throw new IOException("Two files must be selected for this to work");
			}
			
			ImagePlus cfpChannel = new ImagePlus(cfpFile.getAbsolutePath());
			ImagePlus yfpChannel = new ImagePlus(yfpFile.getAbsolutePath());

			ImageSet set = new ImageSet(cfpChannel,yfpChannel);
			
			return set;
		}
		
		public File getFile () {
			JFileChooser chooser = new JFileChooser(FretSawPreferences.getInstance().lastDirectory());
			chooser.setMultiSelectionEnabled(true);
			chooser.setFileFilter(new FileFilter() {
			
				public String getDescription() {
					return "TIFF Files";
				}
				public boolean accept(File f) {
					if (f.isDirectory() || f.getName().toLowerCase().endsWith(".tif") || f.getName().toLowerCase().endsWith(".tiff")) {
						return true;
					}
					return false;
				}
			});
			
			
			int result = chooser.showOpenDialog(this);
			if (result == JFileChooser.CANCEL_OPTION) return null;

			File f = chooser.getSelectedFile();
			FretSawPreferences.getInstance().setLastDirectory(f.getParentFile());
			return f;
		}

		public String fileName () {
			if (cfpFile != null) return cfpFile.getName();
			else return null;
		}
		
		public void actionPerformed(ActionEvent e) {
			
			String c = e.getActionCommand();
			
			if (c.equals("select_cfp")) {
				cfpFile = getFile();
				if (cfpFile != null) {
					cfpField.setText(cfpFile.getName());
				}
				else {
					cfpField.setText(""); 
				}
			}
			else if (c.equals("select_yfp")) {
				yfpFile = getFile();
				if (yfpFile != null) {
					yfpField.setText(yfpFile.getName());
				}
				else {
					yfpField.setText(""); 
				}
			}
			else if (c.equals("cancel")) {
				cancelled = true;
				setVisible(false);
			}
			else if (c.equals("load")) {
				setVisible(false);
			}
			
		}
		
	}
	
}
